{% extends "../base.html" %}

{% block head %}
  <title>Tokit demo</title>
{% end %}

{% block body %}
  <h1>Hello world</h1>

  <p>{* hello *}</p>
  <p>{* app_name name="Tokit" *}</p>
  <p>{* sign_up *}</p>
  <x-boxes></x-boxes>

  <a class=button href="{{ reverse_url('about') }}">Jade-template demo</a>
{% end %}
